#!/bin/python

import os
from math import log, sqrt

# Loading the data

DATADIRS = ["base", "inst"]

def loaddata(dirs):
    for d in dirs:
        for f in os.listdir(d):
            with open(d + "/" + f) as fd:
                for line in fd:
                    yield line.strip().split(",") + [d, f]

DATA = list(loaddata(DATADIRS))

def clean_error(data):
    """Remove records where the request didn't complete successfully."""
    return partition(data, lambda v: v[7] == "true")

GOODDATA, BADDATA = clean_error(DATA)

# Cleaning the data

def clean_intify(data):
    """Parse the elapsed time to an integer."""
    for v in data:
        v[1] = int(v[1])
    return data

def clean_columns(data):
    """
    Extract the interesting columns

    Elapsed time, request type, thread id, base/inst, and experiment id."""
    return [[v[1], v[2], v[5], v[12], v[13]] for v in data]

CLEANDATA = sorted(clean_columns(clean_intify(GOODDATA[:])))

def clean_log(data):
    """Take logs of the elapsed time (correct for log-normality)."""
    for v in data:
        v2 = v[:]
        v2[0] = log(v2[0] + .5)
        yield v2

LOGDATA = list(clean_log(CLEANDATA))

# Drawing histograms---helper functions

def histogram_data(nums, nrange=None, nbuckets=50):
    """
    Bucket data in preparation for drawing a histogram.
    ``nrange`` is pair of min and max values that represents
      the range of values in ``nums``. If ``None``, determined from data.
    ``nbuckets`` is the number of buckets to bucket into.
    """

    if nrange is None:
        nmin, nmax = min(nums), max(nums)
    else:
        nmin, nmax = minmax(nums)
    nspan = nmax - nmin
    buckets = [0 for i in range(nbuckets)]
    for num in nums:
        bucket = int((num - nmin) / (nspan / nbuckets))
        if num == nmax:
            bucket = -1 # special case for largest elt
        buckets[bucket] += 1
    return buckets

def histogram(nums, nrange=None, nbuckets=50, rows=8):
    data = histogram_data(nums, nbuckets=nbuckets, nrange=nrange)
    nmax = max(data)
    data2 = [int(d / nmax * rows) for d in data]
    for r in range(rows):
        for c in range(nbuckets):
            if rows - data2[c] <= r:
                print("#", sep="", end="")
            else:
                print(" ", sep="", end="")
        print()

# Functions for exploring the data

def minmax(x):
    """Returns the smallest and largest value in the data set."""
    x1 = [v[0] for v in x]
    return min(x1), max(x1)

def partition(x, fn):
    """Splits a sequence based on whether ``fn`` is true or false."""
    x1 = []
    x2 = []
    for v in x:
        if fn(v):
            x1.append(v)
        else:
            x2.append(v)
    return x1, x2

def uniq(x, i):
    """Returns the set of values for dimension `i` in the data set."""
    return list(set([v[i] for v in x]))

def select(x, i, t):
    """Selects those data points where axis `i` has value `t`."""
    return [v for v in x if v[i] == t]

def hist(x, *, title="Data", nrange=None):
    """Plots a histogram of the data set."""
    print(title, "N={}".format(len(x)), sep=", ")
    histogram([v[0] for v in x], nrange=nrange)

def split(x, i):
    types = uniq(x, i)
    return {t: select(x, i, t) for t in types}

def splithist(x, i):
    split_x = split(x, i)
    span = minmax(x)
    for cat, sxs in split_x.items():
        hist(sxs, title="[{}]={}".format(i, cat), nrange=span)

def fst(xs):
    return [v[0] for v in xs]

def avg(xs):
    return sum(xs) / len(xs)

def stdev(xs):
    xavg = avg(xs)
    var = sum((x - xavg) ** 2 for x in xs) / (len(xs) - 1)
    return sqrt(var)

def med(xs):
    sxs = sorted(xs)
    if len(xs) % 2 == 1:
        return sxs[len(xs) // 2]
    else:
        return (sxs[len(xs) // 2 - 1] + sxs[len(xs) // 2]) / 2

def analyze(clst):
    ci = fst(select(clst, 3, "inst"))
    cb = fst(select(clst, 3, "base"))
    iavg, bavg = avg(ci), avg(cb)
    istd, bstd = stdev(ci), stdev(cb)
    return iavg - bavg, sqrt(istd**2 / len(ci) + bstd**2 / len(cb))
